/* mp3 decoder with libmad*/

#include <stdio.h>
#include <stdlib.h>
#include <android/log.h>
#include <unistd.h>
#include <jni.h>
#include <assert.h>

#include <TMAudioController.h>
#include <TMRecordDataListener.h>

 
/***************/
class RecordCB : public TMRecordDataListener {
public:
    virtual void onDataLoopStarted(int recordSampleRate, int recordChannels, int recordFormat,
                                       int playSampleRate, int playChannels, int playFormat) {
        __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "onDataLoopStarted: recordSampleRate:%d\n", recordSampleRate);
    }
    virtual void onDataLoopPaused() {
        __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "onDataLoopPaused\n");
    }
    virtual void onRecordDataReady(const char *buf, unsigned int bufsz) {
    #if 0
	    __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "onRecordDataReady\n");
	#endif
        //usleep(80000);
    }
};

class MixCB : public TMRecordDataListener {
public:
    virtual void onDataLoopStarted(int recordSampleRate, int recordChannels, int recordFormat,
                                       int playSampleRate, int playChannels, int playFormat) {
        __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "onDataLoopStarted: playSampleRate:%d\n", playSampleRate);
    }
    virtual void onDataLoopPaused() {
        __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "onDataLoopPaused\n");
    }
    virtual void onRecordDataReady(const char *buf, unsigned int bufsz) {
    #if 0
	    __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "onMixedDataReady\n");
	#endif
        //usleep(80000);
    }
};
/***************/
TMAudioController *controller = NULL;
RecordCB *recordCB = NULL;
MixCB *mixCB = NULL;

void release(JNIEnv *env, jobject cls);
jint init(JNIEnv *env, jobject thiz) {
    controller = new TMAudioController();
    if (controller->probe() != 0) {
        release(env, thiz);
        return -1;
    }
    if (controller->getMode() != TMModeControl) {
        release(env, thiz);
        return -2;
    }
    recordCB = new RecordCB();
    if (controller->registerRecordDataListener(recordCB) != 0) {
        release(env, thiz);
        return -3;
    }
    mixCB = new MixCB();
    if (controller->registerMixedDataListener(mixCB) != 0) {
        release(env, thiz);
        return -4;
    }
    __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "init ok\n");
    return 0;
}

void release(JNIEnv *env, jobject cls) {
    if (controller != NULL)
    {
       if (recordCB != NULL) {
           controller->unregisterRecordDataListener(recordCB); 
           delete recordCB;
           recordCB = NULL;
       }
       if (mixCB != NULL) {
           controller->unregisterMixedDataListener(mixCB);
           delete mixCB;
           mixCB = NULL;
       }
       delete controller;
       controller = NULL;
    }
    __android_log_print(ANDROID_LOG_DEBUG, "TmallAudioRecord", "release ok\n");
}

static JNINativeMethod gMethods[] = {
    {"nativeInit", "()I", (void*)init},
    {"nativeRelease", "()V", (void*)release},
};

static int registerNativeMethods(JNIEnv* env
        , const char* className
        , JNINativeMethod* gMethods, int numMethods) {
    jclass clazz;
    clazz = env->FindClass(className);
    if (clazz == NULL) {
        return JNI_FALSE;
    }
    if (env->RegisterNatives(clazz, gMethods, numMethods) < 0) {
        return JNI_FALSE;
    }

    return JNI_TRUE;
}


static int registerNatives(JNIEnv* env) {
    const char* kClassName = "com/yunos/tv/karaoke/demo/AudioRecord";
    return registerNativeMethods(env, kClassName, gMethods,
            sizeof(gMethods) / sizeof(gMethods[0]));
}

JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM* vm, void* reserved) {
    JNIEnv* env = NULL;
    jint result = -1;

    if (vm->GetEnv((void**) &env, JNI_VERSION_1_4) != JNI_OK) {
        return -1;
    }
    assert(env != NULL);

    if (!registerNatives(env)) {
        return -1;
    }
    result = JNI_VERSION_1_4;

    return result;
}
