/* 
 * Alibaba Group Holding Limited.
 * Copyright (C) 2015 Alibaba Group Holding Limited.
 * ALL RIGHTS RESERVED.
 */

/** @file CommonTypes.h
 * SDK需要用到的一些通用定义。
 *
 */

#ifndef COMMONTYPES_H
#define COMMONTYPES_H


enum TMInputSource {
    TMInputUSB      = 1,        /**< 代表外置USB声卡或者麦克风 */
    TMInputRC       = 2,        /**< 代表自带的遥控器上的麦克风 */
    TMInputMuted    = 3         /**< 代表麦克风打开失败以后使用一个静音的设备 */
};

enum TMOutputSinkValue {
    OutputSinkMin   = 1,
    Hdmi            = 2,        /**< HDMI口输出 */
    LineOut         = 3,        /**< AV口输出  */
    HdmiLineOut     = 4,        /**< HDMI和AV口同时输出 */
    Spdif           = 5,
    HdmiLineAutoOut = 6,       /**< HDMI和AV自动选择输出 */
    OutputSinkMax
};

#if __cplusplus > 199711L
// only defined in c++11
enum class TMOutputSink {
    OutputSinkMin   = 1,
    Hdmi            = 2,        /**< HDMI口输出 */
    LineOut         = 3,        /**< AV口输出  */
    HdmiLineOut     = 4,        /**< HDMI和AV口同时输出 */
    Spdif           = 5,
    HdmiLineAutoOut = 6,       /**< HDMI和AV自动选择输出 */
    OutputSinkMax,
};


inline TMOutputSink outputSinkValue2OutputSink(int value) {
    switch (value) {
        case OutputSinkMin:
            return TMOutputSink::OutputSinkMin;
        case Hdmi:
            return TMOutputSink::Hdmi;
        case LineOut:
            return TMOutputSink::LineOut;
        case HdmiLineOut:
            return TMOutputSink::HdmiLineOut;
        case Spdif:
            return TMOutputSink::Spdif;
        case HdmiLineAutoOut:
            return TMOutputSink::HdmiLineAutoOut;
        case OutputSinkMax:
            return TMOutputSink::OutputSinkMax;
        default:
#ifdef LOGE
            LOGE("outputSinkValue2TMOutputSink unsupported value:%d return hdmi by default.", value);
#endif
            return TMOutputSink::Hdmi;
    }
}


#else
enum TMOutputSink {
    OutputSinkMin   = 1,
    Hdmi            = 2,        /**< HDMI口输出 */
    LineOut         = 3,        /**< AV口输出  */
    HdmiLineOut     = 4,        /**< HDMI和AV口同时输出 */
    Spdif           = 5,
    OutputSinkMax
};
#endif


/** SDK当前运行的模式 */
typedef enum TMControllerMode {

    TMModeControl   = 1, /**< Control Mode: 这个模式下面拥有SDK提供的所有功能。
       	   	   	   	   	   	包括Reverb, Mixing等, 大部分情况下推荐这种模式.
     	 	 	 	 	 */
    TMModeCallback  = 2 /**< 这个模式下Controller只负责从Callback里面拿到PCM数据，和把已经处理好的PCM交给
  	  	  	  	  	  	  	  Controller去写到Audio设备里面。
     	 	 	 	 	 */

} TMControllerMode;


enum TMChannelSetting {
    kChannelStereo    = 1, 
    kChannelLeftOnly  = 2,
    kChannelRightOnly = 3,
};

typedef 
enum TMSampleRate {
    kSampleRate48k    = 48000,
    kSampleRate441k   = 44100,
    kSampleRate32k    = 32000,
    kSampleRate16k    = 16000,
    kSampleRate8k     = 8000
} SampleRate;


enum {
    Error_Success                = 0, /**< 成功 */
    Error_InputDeviceNotAvalible = 101, /**< 输入设备不存在 */
    Error_InputDeviceOpenFailed  = 102, /**< 输入设备打开失败 */
    Error_InputDeviceReadFailed  = 103, /**< 输入设备读取失败， 可能是用户插拔了USB设备， 可能是需要重新插拔USB设备 */
    Error_OutputDeviceNotAvalible = 201, /**< 输出设备不存在 */
};

/** 混音效果的枚举 */
/** @deprecate 使用参数调节，废弃以前模式的方式 */
typedef enum TMAudioFXType {
    kReverbFXKTV = 0,           /**< 默认效果, 模拟普通KTV房的效果 */
    kReverbFXConcert,           /**< 演唱会效果, 模拟演唱会效果 */
    kReverbFXStudio,            /**< 录音房效果, 染音较少, 混响较小 */
    kReverbFXMax,
} TMAudioFXType;

typedef enum TMAudioFXParam {
    kEffectParamReverb = 1      /**< 调节混响的强度属性 */
} TMAudioFXParam;

#endif
