/* 
 * Alibaba Group Holding Limited.
 * Copyright (C) 2015 Alibaba Group Holding Limited.
 * ALL RIGHTS RESERVED.
 */

/** \file TMAudioController.h
 * SDK在Native层的主要接口
 *
 * \version 1.0
 */

#ifndef TMAUDIOCONTROLLER_H
#define TMAUDIOCONTROLLER_H

#include "CommonTypes.h"

class TMAudioEffect;
class TMAudioDataCallback;
class TMRecordDataListener;

/**
   SDK 在native层的控制接口, 主要是提供native的一些接口和回调, 比如Data回调和效果.

   这个类在调用之前要先调用 KaraokeController 来进行初始化底层相关, 否则无法正常工作.
 */
class TMAudioController {
public:
    /** 构造函数 */
    TMAudioController();

    /**
     * 探测是否已经SDK已经成功初始化
     * @return －1表示失败， 0 表示成功连接到SDK底层
     */
    int probe();

    /**
     * 得到当前运行的mode.
     * @return  返回当前运行的模式。如果未probe， 返回结果随机。
     * @see TMControllerMode
     */
    TMControllerMode getMode();

    /**
     * 注册DataCallback
     */
    int registerDataCallback(TMAudioDataCallback *callback);

    /**
     * 注销DataCallback
     */
    int unregisterDataCallback(TMAudioDataCallback *callback);


    /**
     * 注册人声录音数据Listener， 人声是未加任何效果的.
     */
    int registerRecordDataListener(TMRecordDataListener *recordListener);

    /**
     * 注销人声录音数据Listener
     */
    int unregisterRecordDataListener(TMRecordDataListener *recordListener);

    /**
     * 注册混音好的数据Listener， 可以用来录制最后的声音.
     */
    int registerMixedDataListener(TMRecordDataListener *recordListener);

    /**
     * 注销混音好的数据Listener
     */
    int unregisterMixedDataListener(TMRecordDataListener *recordListener);
    
    /**
     * 注册声音效果, 这个效果主要是增加Reverb效果
     *
     * 这个效果会应用到人声上面, 伴奏音不会走这里.
     * reverb的效果现在只有几种, 详见#TMAudioFXType.
     * 效果类是在#TMAudioEffect接口,
     * 在不注册任何效果的情况下, 会使用内置的默认音效处理
     *
     */
    int registerVoiceEffect(TMAudioEffect *effect);

    /**
     * 注销声音效果处理器
     * 去掉这个效果以后, 会使用内置的默认音效处理.
     */
    int unregisterVoiceEffect(TMAudioEffect *effect);
};


#endif
