/* 
 * Alibaba Group Holding Limited.
 * Copyright (C) 2015 Alibaba Group Holding Limited.
 * ALL RIGHTS RESERVED.
 */

/**
   @file TMAudioEffect.h
   音效的接口， 可以通过实现这个接口来自定义一个音效。
   @version 1.0
*/

#ifndef AUDIOEFFECT_H
#define AUDIOEFFECT_H

#include <unistd.h>


#include "CommonTypes.h"

typedef enum {
    TMEffectTuneReverbLevel,
} TMEffectTuneType;

class  AudioEffectTTDT;

/**
 * 自定义声音效果处理的函数
 */
class TMAudioEffect {
protected:
    TMSampleRate sampleRate_;
    int inChannel_;
    int outChannel_;
    TMAudioFXType effect_;
public:
    TMAudioEffect();
    virtual ~TMAudioEffect();

    /**
     * 初始化AudioEffect的接口，子类需要在这个函数里面实现一些相应的初始化工作，
     * @param sampleRate 采样率
     * @param inChannel 数据谁人的声道
     * @param outChannel 数据输出的声道，通常是和inChannel相等
     * @param effect 初始音效
     */
    virtual int init(TMSampleRate sampleRate, int inChannel, int outChannel,  TMAudioFXType effect) = 0;

    /**
     * 打开和关闭该音效处理接口， SDK会在需要的时候打开音效活着关闭音效
     * @param enable 打开／关闭
     */
    virtual int setEnable(bool enable) = 0;

    /** 
     * 处理buffer的接口， 子类应该在这个接口中声音进行处理。
     * 
     * 需要注意的是， inbuf, 和 outfb 有可能是同一个指针。 处理函数里面需要做这样的假设。
     *
     * @param inbuf 输入数据的buffer指针
     * @param insz  输入buffer的大小
     * @param outbf 输出buffer的大小, buffer由调用者分配
     * @param outsz 实际填充的buffer size
     */
    virtual int  process(char *inbuf, size_t insz, char *outbf, size_t &outsz) = 0;

    /**
     * 切换新的效果， 这时候的效果
     * @deprecate 使用changeEffetParam
     */
    virtual int  changeEffect(TMAudioFXType effect) = 0;

    /**
     * 调节效果param的参数的值到val
     */
    virtual int changeEffectParam(TMAudioFXParam param, int val) = 0;

    /**
     * 得到参数现在的值, return 0 means success, -1 means error.
     */
    virtual int getEffectParam(TMAudioFXParam param, int& rval) = 0;

    /**
     * 得到参数的最大最小值范围
     */
    virtual int getRangeOfParam(TMAudioFXParam param, int& min, int& max) = 0;

};

#endif
