/* 
 * Alibaba Group Holding Limited.
 * Copyright (C) 2015 Alibaba Group Holding Limited.
 * ALL RIGHTS RESERVED.
 */

/** \file TMRecordDataListener.h
 * 在普通模式下注册录音数据的callback
 *
 * \version 1.0
 */


#ifndef TMRECORDDATALISTENER_H
#define TMRECORDDATALISTENER_H

#include "CommonUtils.h"


/**
 * 这个Listener主要是在录音数据准备好之后调用， 只在普通模式下面有效。
 *
 * 下面是一个使用的例子：
 * \code{.cpp}
class RecordCB : public TMRecordDataListener {
public:
        virtual void onDataLoopStarted(int recordSampleRate, int recordChannels, int recordFormat,
                                       int playSampleRate, int playChannels, int playFormat)
    {
        printf("onDataLoopStarted: recordChannels:%d\n", recordChannels);
    }

    virtual void onDataLoopPaused()
    {
        printf("onDataLoopPaused\n");

    }

    virtual void onRecordDataReady(const char *buf, unsigned int bufsz)
    {
        printf("onRecordDataReady: buf: %p size:%d\n", buf, bufsz);
    }
};

TMAudioController *controller = new TMAudioController();
assert(controller->probe() == 0);
assert(controller->getMode() == TMModeControl);
controller->registerRecordDataListener(new RecordCB);

 * \endcode
 */
class TMRecordDataListener {
public:
	/**
	 * 这个函数会在混音线程开始混音的时候调到
	 * 参数分别是录音以及放音设备的一些属性， 会在混音线程开始活着恢复运行的时候调用。
	 * @param recordSampleRate registerRecordDataListener注册的回调使用recordSampleRate提供的samle Rate和其他参数; registerMixedDataListener注册的回调使用playSampleRate，以及其他参数。
	 */
    virtual void onDataLoopStarted(int recordSampleRate, int recordChannels, int recordFormat,
			int playSampleRate, int playChannels, int playFormat) = 0;

    /**
     * 这个函数会在混音线程暂停时候被调用
     * 被调用表示混音线程因为某种原因，也许是系统需要Audio设备之类的原因暂停。
     */
	virtual void onDataLoopPaused() = 0;

	/**
	 * 录音数据的回调
	 * 这个函数会在数据准备好以后被调用， 这个函数会在回调线程上调用，不是在混音主线程上。
	 */
    virtual void onRecordDataReady(const char *buf, unsigned int bufsz) = 0;
};

#endif
